searchData={"items":[{"type":"module","title":"AI","doc":"AI is an Elixir SDK for building AI-powered applications.\n\nIt provides a unified API to interact with various AI model providers\nlike OpenAI, Anthropic, and others.","ref":"AI.html"},{"type":"function","title":"AI.generate_text/1","doc":"Generates text using an AI model.","ref":"AI.html#generate_text/1"},{"type":"function","title":"Options - AI.generate_text/1","doc":"* `:model` - The language model to use\n  * `:system` - A system message that will be part of the prompt\n  * `:prompt` - A simple text prompt (can use either prompt or messages)\n  * `:messages` - A list of messages (can use either prompt or messages)\n  * `:max_tokens` - Maximum number of tokens to generate\n  * `:temperature` - Temperature setting for randomness\n  * `:top_p` - Nucleus sampling\n  * `:tools` - Tools that are accessible to and can be called by the model\n  * `:tool_choice` - The tool choice strategy (default: 'auto')","ref":"AI.html#generate_text/1-options"},{"type":"function","title":"Examples - AI.generate_text/1","doc":"{:ok, result} = AI.generate_text(%{\n      model: AI.openai_compatible(\"gpt-3.5-turbo\", base_url: \"https://api.example.com\"),\n      system: \"You are a friendly assistant!\",\n      prompt: \"Why is the sky blue?\"\n    })\n\n    IO.puts(result.text)","ref":"AI.html#generate_text/1-examples"},{"type":"function","title":"AI.openai/2","doc":"Creates an OpenAI model with the specified model ID.\n\nThis function creates a model that uses the official OpenAI API.","ref":"AI.html#openai/2"},{"type":"function","title":"Options - AI.openai/2","doc":"* `:api_key` - The API key to use for authentication (default: OPENAI_API_KEY environment variable)\n  * `:base_url` - The base URL of the API (default: \"https://api.openai.com\")\n  * `:structured_outputs` - Whether the model supports structured outputs (default: false)\n  * `:use_legacy_function_calling` - Whether to use legacy function calling format (default: false)\n  * `:reasoning_effort` - For O-series models (o1, o3), controls the reasoning effort (low, medium, high)","ref":"AI.html#openai/2-options"},{"type":"function","title":"Examples - AI.openai/2","doc":"model = AI.openai(\"gpt-4\")\n    \n    # With custom API key\n    model = AI.openai(\"gpt-4\", api_key: \"your-api-key\")\n    \n    # With structured outputs\n    model = AI.openai(\"gpt-4\", structured_outputs: true)\n    \n    # With reasoning effort for O-series models\n    model = AI.openai(\"o1-mini\", reasoning_effort: \"high\")","ref":"AI.html#openai/2-examples"},{"type":"function","title":"AI.openai_compatible/2","doc":"Creates an OpenAI-compatible provider with the specified model ID.\n\nThis function creates a model that can be used with OpenAI-compatible APIs,\nsuch as Ollama, LMStudio, and any other API that follows the OpenAI format.","ref":"AI.html#openai_compatible/2"},{"type":"function","title":"Options - AI.openai_compatible/2","doc":"* `:base_url` - The base URL of the API (required)\n  * `:api_key` - The API key to use for authentication (optional)\n  * `:headers` - Additional headers to include in requests (optional)\n  * `:supports_image_urls` - Whether the model supports image URLs (default: false)\n  * `:supports_structured_outputs` - Whether the model supports structured outputs (default: false)","ref":"AI.html#openai_compatible/2-options"},{"type":"function","title":"Examples - AI.openai_compatible/2","doc":"model = AI.openai_compatible(\"gpt-3.5-turbo\", base_url: \"https://api.example.com\")\n    \n    # With API key\n    model = AI.openai_compatible(\"gpt-4\", \n      base_url: \"https://api.openai.com\", \n      api_key: System.get_env(\"OPENAI_API_KEY\")\n    )","ref":"AI.html#openai_compatible/2-examples"},{"type":"function","title":"AI.openai_completion/2","doc":"Creates an OpenAI completion model with the specified model ID.\n\nThis function creates a model that uses the official OpenAI completion API.","ref":"AI.html#openai_completion/2"},{"type":"function","title":"Options - AI.openai_completion/2","doc":"* `:api_key` - The API key to use for authentication (default: OPENAI_API_KEY environment variable)\n  * `:base_url` - The base URL of the API (default: \"https://api.openai.com\")","ref":"AI.html#openai_completion/2-options"},{"type":"function","title":"Examples - AI.openai_completion/2","doc":"model = AI.openai_completion(\"text-davinci-003\")\n    \n    # With custom API key\n    model = AI.openai_completion(\"text-davinci-003\", api_key: \"your-api-key\")","ref":"AI.html#openai_completion/2-examples"},{"type":"function","title":"AI.stream_text/1","doc":"Streams text generation from an AI model, returning chunks as they are generated.","ref":"AI.html#stream_text/1"},{"type":"function","title":"Options - AI.stream_text/1","doc":"* `:model` - The language model to use\n  * `:system` - A system message that will be part of the prompt\n  * `:prompt` - A simple text prompt (can use either prompt or messages)\n  * `:messages` - A list of messages (can use either prompt or messages)\n  * `:max_tokens` - Maximum number of tokens to generate\n  * `:temperature` - Temperature setting for randomness\n  * `:top_p` - Nucleus sampling\n  * `:top_k` - Top-k sampling\n  * `:frequency_penalty` - Penalize new tokens based on their frequency\n  * `:presence_penalty` - Penalize new tokens based on their presence\n  * `:tools` - Tools that are accessible to and can be called by the model\n  * `:mode` - Return format: `:string` (default) for plain text chunks, or `:event` for event tuples","ref":"AI.html#stream_text/1-options"},{"type":"function","title":"Examples - AI.stream_text/1","doc":"","ref":"AI.html#stream_text/1-examples"},{"type":"function","title":"Default String Mode - AI.stream_text/1","doc":"{:ok, result} = AI.stream_text(%{\n      model: AI.openai(\"gpt-3.5-turbo\"),\n      system: \"You are a friendly assistant!\",\n      prompt: \"Why is the sky blue?\"\n    })\n\n    # Process chunks as they arrive - each chunk is a string\n    result.stream\n    |> Stream.each(&IO.write/1)\n    |> Stream.run()\n\n    # Or collect all chunks into a single string\n    full_text = Enum.join(result.stream, \"\")","ref":"AI.html#stream_text/1-default-string-mode"},{"type":"function","title":"Event Mode - AI.stream_text/1","doc":"{:ok, result} = AI.stream_text(%{\n      model: AI.openai(\"gpt-3.5-turbo\"),\n      prompt: \"Tell me a story\",\n      mode: :event\n    })\n\n    # Process different event types\n    result.stream\n    |> Enum.reduce(\"\", fn\n      {:text_delta, chunk}, acc ->\n        IO.write(chunk)\n        acc <> chunk\n        \n      {:finish, reason}, acc ->\n        IO.puts(\"\\nFinished: #{reason}\")\n        acc\n        \n      {:error, error}, acc ->\n        IO.puts(\"\\nError: #{inspect(error)}\")\n        acc\n        \n      _, acc -> acc\n    end)","ref":"AI.html#stream_text/1-event-mode"},{"type":"module","title":"AI.Core.GenerateText","doc":"Core implementation for generating text using language models.\n\nThis module handles the actual text generation functionality, including:\n- Processing the prompt and messages\n- Handling tool calls\n- Processing the model response","ref":"AI.Core.GenerateText.html"},{"type":"function","title":"AI.Core.GenerateText.generate_text/1","doc":"Generate text using a language model.\n\nThis is the core implementation that AI.generate_text/1 calls.","ref":"AI.Core.GenerateText.html#generate_text/1"},{"type":"module","title":"AI.Core.MockLanguageModel","doc":"A mock language model implementation for testing.\n\nThis module allows you to create mock model instances with predefined behaviors\nfor testing various AI.generate_text and AI.stream_text scenarios.","ref":"AI.Core.MockLanguageModel.html"},{"type":"function","title":"AI.Core.MockLanguageModel.do_generate/2","doc":"Mock implementation of the generate text function.\n\nBy default, returns \"Hello, world!\" if no custom implementation is provided.","ref":"AI.Core.MockLanguageModel.html#do_generate/2"},{"type":"function","title":"AI.Core.MockLanguageModel.do_stream/2","doc":"Mock implementation of the stream text function.\n\nBy default, streams \"Hello, world!\" if no custom implementation is provided.","ref":"AI.Core.MockLanguageModel.html#do_stream/2"},{"type":"function","title":"AI.Core.MockLanguageModel.new/1","doc":"Creates a new mock language model with the given options.","ref":"AI.Core.MockLanguageModel.html#new/1"},{"type":"function","title":"Options - AI.Core.MockLanguageModel.new/1","doc":"* `:do_generate` - A function that will be called when `do_generate/1` is invoked\n  * `:do_stream` - A function that will be called when `do_stream/1` is invoked\n  * `:supports_image_urls` - Whether the model supports image URLs (default: false)\n  * `:supports_structured_outputs` - Whether the model supports structured outputs (default: false)\n  * `:supports_url` - A function that checks if a URL is supported","ref":"AI.Core.MockLanguageModel.html#new/1-options"},{"type":"module","title":"AI.Core.StreamText","doc":"Core implementation for streaming text from language models.\n\nThis module handles the streaming functionality, including:\n- Processing the prompt and messages\n- Setting up the stream\n- Handling streaming events from the model","ref":"AI.Core.StreamText.html"},{"type":"function","title":"AI.Core.StreamText.stream_text/1","doc":"Stream text from a language model.\n\nThis is the core implementation that AI.stream_text/1 calls.","ref":"AI.Core.StreamText.html#stream_text/1"},{"type":"behaviour","title":"AI.Provider.LanguageModelV1","doc":"Behaviour for language model providers.","ref":"AI.Provider.LanguageModelV1.html"},{"type":"callback","title":"AI.Provider.LanguageModelV1.default_object_generation_mode/1","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:default_object_generation_mode/1"},{"type":"callback","title":"AI.Provider.LanguageModelV1.do_generate/2","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:do_generate/2"},{"type":"callback","title":"AI.Provider.LanguageModelV1.do_stream/2","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:do_stream/2"},{"type":"callback","title":"AI.Provider.LanguageModelV1.provider/1","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:provider/1"},{"type":"callback","title":"AI.Provider.LanguageModelV1.specification_version/0","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:specification_version/0"},{"type":"callback","title":"AI.Provider.LanguageModelV1.supports_image_urls?/1","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:supports_image_urls?/1"},{"type":"callback","title":"AI.Provider.LanguageModelV1.supports_structured_outputs?/1","doc":"","ref":"AI.Provider.LanguageModelV1.html#c:supports_structured_outputs?/1"},{"type":"module","title":"AI.Provider.Utils.EventSource","doc":"Utility functions for handling Server-Sent Events (SSE).\n\nThis module provides functionality for making HTTP requests that receive\nServer-Sent Events (SSE) responses, commonly used for streaming API responses\nfrom AI providers like OpenAI.\n\nThe EventSource module handles:\n\n* Creating HTTP connections with proper SSE headers\n* Parsing SSE format (data:, event:, id:, retry: fields)\n* Converting SSE events into structured Elixir stream events\n* Supporting backpressure for efficient streaming\n* Error handling and connection cleanup","ref":"AI.Provider.Utils.EventSource.html"},{"type":"function","title":"AI.Provider.Utils.EventSource.post/4","doc":"Makes a POST request to the specified URL with streaming response.\n\nThis function sends a POST request with the provided body and headers to the URL,\nand returns a structure that can be used to stream the SSE responses. The returned\nstream emits events of type `stream_event()`.","ref":"AI.Provider.Utils.EventSource.html#post/4"},{"type":"function","title":"Parameters - AI.Provider.Utils.EventSource.post/4","doc":"* `url` - The URL to make the request to\n  * `body` - The JSON request body\n  * `headers` - HTTP headers to include in the request\n  * `options` - Additional options for the request:\n      * `:timeout` - Connection timeout in milliseconds (default: 30000)\n      * `:max_line_length` - Maximum length of an SSE line (default: 16384)\n      * `:retry_interval` - Time to wait before reconnecting in ms (default: 3000)","ref":"AI.Provider.Utils.EventSource.html#post/4-parameters"},{"type":"function","title":"Returns - AI.Provider.Utils.EventSource.post/4","doc":"* `{:ok, response}` - Where response contains:\n    * `status` - HTTP status code\n    * `body` - Response body (usually just initialization message)\n    * `stream` - Stream of `stream_event()` events\n  * `{:error, reason}` - If the request fails","ref":"AI.Provider.Utils.EventSource.html#post/4-returns"},{"type":"function","title":"Examples - AI.Provider.Utils.EventSource.post/4","doc":"{:ok, response} = EventSource.post(\n      \"https://api.openai.com/v1/chat/completions\",\n      %{model: \"gpt-3.5-turbo\", messages: [%{role: \"user\", content: \"Hello\"}], stream: true},\n      %{\"Authorization\" => \"Bearer my-api-key\", \"Content-Type\" => \"application/json\"},\n      %{timeout: 60_000}\n    )\n\n    response.stream\n    |> Stream.each(fn\n      {:text_delta, chunk} -> IO.write(chunk)\n      {:finish, reason} -> IO.puts(\"\\nFinished: #{reason}\")\n      {:error, error} -> IO.puts(\"Error: #{inspect(error)}\")\n      _ -> :ok\n    end)\n    |> Stream.run()","ref":"AI.Provider.Utils.EventSource.html#post/4-examples"},{"type":"type","title":"AI.Provider.Utils.EventSource.response/0","doc":"Response from an SSE request\n\n* `{:ok, response}` - Successful response with status, body, and stream\n* `{:error, reason}` - Error occurred during connection or streaming","ref":"AI.Provider.Utils.EventSource.html#t:response/0"},{"type":"type","title":"AI.Provider.Utils.EventSource.stream_event/0","doc":"Stream events that can be emitted by an SSE stream\n\n* `{:text_delta, text}` - A chunk of text from the model\n* `{:finish, reason}` - Stream has completed with reason (e.g., \"stop\", \"length\")\n* `{:metadata, data}` - Additional metadata from the model\n* `{:error, reason}` - An error occurred during streaming","ref":"AI.Provider.Utils.EventSource.html#t:stream_event/0"},{"type":"module","title":"AI.Provider.Utils.Headers","doc":"Utility functions for handling HTTP headers.","ref":"AI.Provider.Utils.Headers.html"},{"type":"function","title":"AI.Provider.Utils.Headers.combine/2","doc":"","ref":"AI.Provider.Utils.Headers.html#combine/2"},{"type":"module","title":"AI.Provider.Utils.JsonApi","doc":"Utility functions for making JSON API requests.","ref":"AI.Provider.Utils.JsonApi.html"},{"type":"function","title":"AI.Provider.Utils.JsonApi.post/4","doc":"Makes a POST request to a JSON API endpoint.","ref":"AI.Provider.Utils.JsonApi.html#post/4"},{"type":"function","title":"Parameters - AI.Provider.Utils.JsonApi.post/4","doc":"* `url` - The URL to send the request to\n  * `body` - The request body (will be converted to JSON)\n  * `headers` - Map of HTTP headers\n  * `options` - Additional options","ref":"AI.Provider.Utils.JsonApi.html#post/4-parameters"},{"type":"function","title":"Returns - AI.Provider.Utils.JsonApi.post/4","doc":"* `{:ok, response}` - The successful response\n  * `{:error, reason}` - The error reason","ref":"AI.Provider.Utils.JsonApi.html#post/4-returns"},{"type":"type","title":"AI.Provider.Utils.JsonApi.response/0","doc":"","ref":"AI.Provider.Utils.JsonApi.html#t:response/0"},{"type":"module","title":"AI.ProviderRegistry","doc":"A registry for AI providers and their language models.\n\nThis module provides a central registry for managing different AI providers\nand their associated language models, enabling dynamic registration and lookup.","ref":"AI.ProviderRegistry.html"},{"type":"function","title":"AI.ProviderRegistry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AI.ProviderRegistry.html#child_spec/1"},{"type":"function","title":"AI.ProviderRegistry.get_language_model/2","doc":"Gets a language model by its model ID string.\n\nModel IDs can be in the format \"provider:model\" to specify which provider should\nhandle the model, e.g., \"openai:gpt-4\" or \"anthropic:claude-3-sonnet\".","ref":"AI.ProviderRegistry.html#get_language_model/2"},{"type":"function","title":"Parameters - AI.ProviderRegistry.get_language_model/2","doc":"* `model_id` - A string identifying the model, in format \"provider:model_name\"\n  * `opts` - Optional configuration for the model","ref":"AI.ProviderRegistry.html#get_language_model/2-parameters"},{"type":"function","title":"Returns - AI.ProviderRegistry.get_language_model/2","doc":"* `{:ok, language_model}` - The initialized language model\n  * `{:error, reason}` - If the model or provider could not be found","ref":"AI.ProviderRegistry.html#get_language_model/2-returns"},{"type":"function","title":"AI.ProviderRegistry.get_provider/1","doc":"Retrieves a provider by its ID.","ref":"AI.ProviderRegistry.html#get_provider/1"},{"type":"function","title":"Parameters - AI.ProviderRegistry.get_provider/1","doc":"* `provider_id` - The string ID of the provider to get","ref":"AI.ProviderRegistry.html#get_provider/1-parameters"},{"type":"function","title":"Returns - AI.ProviderRegistry.get_provider/1","doc":"* `{:ok, provider}` - The provider with its module and options\n  * `{:error, :not_found}` - If no provider with that ID is registered","ref":"AI.ProviderRegistry.html#get_provider/1-returns"},{"type":"function","title":"AI.ProviderRegistry.register/3","doc":"Registers a provider module with the given ID.","ref":"AI.ProviderRegistry.html#register/3"},{"type":"function","title":"Parameters - AI.ProviderRegistry.register/3","doc":"* `provider_id` - A string ID for the provider\n  * `provider_module` - The module that implements the provider\n  * `opts` - Optional configuration for the provider","ref":"AI.ProviderRegistry.html#register/3-parameters"},{"type":"function","title":"Returns - AI.ProviderRegistry.register/3","doc":"* `:ok` - If registration was successful\n  * `{:error, reason}` - If registration failed","ref":"AI.ProviderRegistry.html#register/3-returns"},{"type":"function","title":"AI.ProviderRegistry.start_link/1","doc":"","ref":"AI.ProviderRegistry.html#start_link/1"},{"type":"module","title":"AI.Providers.OpenAI.ChatLanguageModel","doc":"Implementation of the OpenAI chat language model.","ref":"AI.Providers.OpenAI.ChatLanguageModel.html"},{"type":"function","title":"AI.Providers.OpenAI.ChatLanguageModel.handle_stream_response/4","doc":"","ref":"AI.Providers.OpenAI.ChatLanguageModel.html#handle_stream_response/4"},{"type":"function","title":"AI.Providers.OpenAI.ChatLanguageModel.new/3","doc":"Creates a new OpenAI chat language model instance.","ref":"AI.Providers.OpenAI.ChatLanguageModel.html#new/3"},{"type":"type","title":"AI.Providers.OpenAI.ChatLanguageModel.t/0","doc":"","ref":"AI.Providers.OpenAI.ChatLanguageModel.html#t:t/0"},{"type":"module","title":"AI.Providers.OpenAI.ChatMessages","doc":"Converts messages to OpenAI chat format, handling various message types,\nroles, and content formats.","ref":"AI.Providers.OpenAI.ChatMessages.html"},{"type":"function","title":"AI.Providers.OpenAI.ChatMessages.convert_to_openai_chat_messages/2","doc":"Converts messages to OpenAI chat format.","ref":"AI.Providers.OpenAI.ChatMessages.html#convert_to_openai_chat_messages/2"},{"type":"function","title":"Options - AI.Providers.OpenAI.ChatMessages.convert_to_openai_chat_messages/2","doc":"* `:use_legacy_function_calling` - Whether to use legacy function calling format (default: false)\n  * `:system_message_mode` - How to handle system messages (:system, :developer, or :remove)","ref":"AI.Providers.OpenAI.ChatMessages.html#convert_to_openai_chat_messages/2-options"},{"type":"type","title":"AI.Providers.OpenAI.ChatMessages.message/0","doc":"","ref":"AI.Providers.OpenAI.ChatMessages.html#t:message/0"},{"type":"type","title":"AI.Providers.OpenAI.ChatMessages.system_message_mode/0","doc":"","ref":"AI.Providers.OpenAI.ChatMessages.html#t:system_message_mode/0"},{"type":"type","title":"AI.Providers.OpenAI.ChatMessages.warning/0","doc":"","ref":"AI.Providers.OpenAI.ChatMessages.html#t:warning/0"},{"type":"module","title":"AI.Providers.OpenAI.CompletionLanguageModel","doc":"Implementation of the OpenAI completion language model.","ref":"AI.Providers.OpenAI.CompletionLanguageModel.html"},{"type":"function","title":"AI.Providers.OpenAI.CompletionLanguageModel.new/1","doc":"Creates a new OpenAI completion language model instance with default settings.","ref":"AI.Providers.OpenAI.CompletionLanguageModel.html#new/1"},{"type":"function","title":"AI.Providers.OpenAI.CompletionLanguageModel.new/3","doc":"Creates a new OpenAI completion language model instance with custom settings and config.","ref":"AI.Providers.OpenAI.CompletionLanguageModel.html#new/3"},{"type":"type","title":"AI.Providers.OpenAI.CompletionLanguageModel.t/0","doc":"","ref":"AI.Providers.OpenAI.CompletionLanguageModel.html#t:t/0"},{"type":"module","title":"AI.Providers.OpenAICompatible.ChatLanguageModel","doc":"OpenAI-compatible chat language model implementation.\n\nThis module implements the language model interface for OpenAI-compatible APIs.","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html"},{"type":"function","title":"AI.Providers.OpenAICompatible.ChatLanguageModel.do_generate/2","doc":"Generate text using the OpenAI-compatible API.","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html#do_generate/2"},{"type":"function","title":"Options - AI.Providers.OpenAICompatible.ChatLanguageModel.do_generate/2","doc":"* `:messages` - List of messages to send to the API (required)\n  * `:temperature` - Temperature for sampling (default: 1.0)\n  * `:max_tokens` - Maximum number of tokens to generate (optional)\n  * `:tools` - List of tools available to the model (optional)\n  * `:tool_choice` - Tool choice configuration (optional)\n  * `:response_format` - Response format configuration (optional)","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html#do_generate/2-options"},{"type":"function","title":"AI.Providers.OpenAICompatible.ChatLanguageModel.do_stream/2","doc":"Stream text generation using the OpenAI-compatible API.\n\nThis function enables streaming responses from the model, returning chunks\nas they are generated rather than waiting for the complete response.","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html#do_stream/2"},{"type":"function","title":"Options - AI.Providers.OpenAICompatible.ChatLanguageModel.do_stream/2","doc":"* `:messages` - List of messages to send to the API (required)\n  * `:temperature` - Temperature for sampling (default: 1.0)\n  * `:max_tokens` - Maximum number of tokens to generate (optional)\n  * `:tools` - List of tools available to the model (optional)\n  * `:tool_choice` - Tool choice configuration (optional)\n  * `:response_format` - Response format configuration (optional)","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html#do_stream/2-options"},{"type":"function","title":"AI.Providers.OpenAICompatible.ChatLanguageModel.new/2","doc":"Creates a new OpenAI-compatible chat language model.","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html#new/2"},{"type":"function","title":"Options - AI.Providers.OpenAICompatible.ChatLanguageModel.new/2","doc":"* `:model_id` - The model ID to use (required)\n  * `:supports_image_urls` - Whether the model supports image URLs (default: false)\n  * `:supports_structured_outputs` - Whether the model supports structured outputs (default: false)","ref":"AI.Providers.OpenAICompatible.ChatLanguageModel.html#new/2-options"},{"type":"module","title":"AI.Providers.OpenAICompatible.MessageConversion","doc":"Utilities for converting between AI.Core message formats and OpenAI API formats.","ref":"AI.Providers.OpenAICompatible.MessageConversion.html"},{"type":"function","title":"AI.Providers.OpenAICompatible.MessageConversion.convert_to_openai_compatible_chat_messages/1","doc":"Converts AI.Core messages to OpenAI-compatible chat messages format.","ref":"AI.Providers.OpenAICompatible.MessageConversion.html#convert_to_openai_compatible_chat_messages/1"},{"type":"module","title":"AI.Providers.OpenAICompatible.Provider","doc":"Provider implementation for OpenAI-compatible APIs.\n\nThis provider can be used to interact with any API that is compatible with the OpenAI API,\nsuch as Ollama, LMStudio, and others.","ref":"AI.Providers.OpenAICompatible.Provider.html"},{"type":"function","title":"AI.Providers.OpenAICompatible.Provider.new/1","doc":"Creates a new OpenAI-compatible provider with the given options.","ref":"AI.Providers.OpenAICompatible.Provider.html#new/1"},{"type":"function","title":"Options - AI.Providers.OpenAICompatible.Provider.new/1","doc":"* `:base_url` - The base URL of the API (required)\n  * `:name` - The name of the provider (default: \"openai-compatible\")\n  * `:api_key` - The API key to use for authentication (optional)\n  * `:headers` - Additional headers to include in requests (optional)","ref":"AI.Providers.OpenAICompatible.Provider.html#new/1-options"},{"type":"module","title":"AI.Stream.Event","doc":"Defines standard event types for streaming interactions with AI models.\n\nThese structured event types represent different kinds of data that can be\nemitted during a streaming interaction, allowing for standardized handling\nacross different providers.","ref":"AI.Stream.Event.html"},{"type":"function","title":"AI.Stream.Event.from_tuple/1","doc":"Converts a tuple format to an Event struct.","ref":"AI.Stream.Event.html#from_tuple/1"},{"type":"function","title":"Examples - AI.Stream.Event.from_tuple/1","doc":"iex> AI.Stream.Event.from_tuple({:text_delta, \"Hello\"})\n    %AI.Stream.Event.TextDelta{content: \"Hello\"}","ref":"AI.Stream.Event.html#from_tuple/1-examples"},{"type":"type","title":"AI.Stream.Event.t/0","doc":"","ref":"AI.Stream.Event.html#t:t/0"},{"type":"function","title":"AI.Stream.Event.to_tuple/1","doc":"Converts an Event struct to a tuple format.","ref":"AI.Stream.Event.html#to_tuple/1"},{"type":"function","title":"Examples - AI.Stream.Event.to_tuple/1","doc":"iex> event = %AI.Stream.Event.TextDelta{content: \"Hello\"}\n    iex> AI.Stream.Event.to_tuple(event)\n    {:text_delta, \"Hello\"}","ref":"AI.Stream.Event.html#to_tuple/1-examples"},{"type":"type","title":"AI.Stream.Event.tuple_format/0","doc":"","ref":"AI.Stream.Event.html#t:tuple_format/0"},{"type":"module","title":"AI.Stream.Event.Error","doc":"Represents an error that occurred during streaming.","ref":"AI.Stream.Event.Error.html"},{"type":"type","title":"AI.Stream.Event.Error.t/0","doc":"","ref":"AI.Stream.Event.Error.html#t:t/0"},{"type":"module","title":"AI.Stream.Event.Finish","doc":"Represents the completion of a stream with a reason.","ref":"AI.Stream.Event.Finish.html"},{"type":"type","title":"AI.Stream.Event.Finish.t/0","doc":"","ref":"AI.Stream.Event.Finish.html#t:t/0"},{"type":"module","title":"AI.Stream.Event.Metadata","doc":"Represents metadata information from the model.","ref":"AI.Stream.Event.Metadata.html"},{"type":"type","title":"AI.Stream.Event.Metadata.t/0","doc":"","ref":"AI.Stream.Event.Metadata.html#t:t/0"},{"type":"module","title":"AI.Stream.Event.TextDelta","doc":"Represents a chunk of text from the model.","ref":"AI.Stream.Event.TextDelta.html"},{"type":"type","title":"AI.Stream.Event.TextDelta.t/0","doc":"","ref":"AI.Stream.Event.TextDelta.html#t:t/0"},{"type":"module","title":"AI.Stream.Event.ToolCall","doc":"Represents a tool call from the model.","ref":"AI.Stream.Event.ToolCall.html"},{"type":"type","title":"AI.Stream.Event.ToolCall.t/0","doc":"","ref":"AI.Stream.Event.ToolCall.html#t:t/0"},{"type":"module","title":"AI.Stream.OpenAICompatibleTransformer","doc":"Stream transformer for OpenAI-compatible API streams.\n\nThis transformer handles streams from OpenAI-compatible APIs like LMStudio or other\nopen-source/self-hosted models that implement the OpenAI API specification but might\nhave slight differences in event formatting.","ref":"AI.Stream.OpenAICompatibleTransformer.html"},{"type":"module","title":"AI.Stream.OpenAITransformer","doc":"Stream transformer for OpenAI-compatible streams.\n\nThis transformer handles the specific format of events emitted by the OpenAI Chat API\nand its compatible implementations, converting them to standardized AI.Stream.Event structs.\n\nFeatures:\n- Converts raw events to standardized Event structs\n- Filters out unnecessary metadata chunks\n- Maintains proper order of text chunks","ref":"AI.Stream.OpenAITransformer.html"},{"type":"behaviour","title":"AI.Stream.Transformer","doc":"Behaviour for stream transformers that convert provider-specific streams to standardized events.\n\nStream transformers are responsible for:\n1. Taking provider-specific streaming events\n2. Converting them into standardized AI.Stream.Event structs\n3. Handling provider-specific event formats and quirks\n\nThis separation of concerns allows provider modules to focus on API communication\nwhile stream transformers handle the interpretation of streaming data formats.","ref":"AI.Stream.Transformer.html"},{"type":"function","title":"AI.Stream.Transformer.convert_tuple_to_event/1","doc":"Utility function to convert basic tuple event formats to AI.Stream.Event structs.\n\nThis helper can be used by transformer implementations to simplify conversion\nof simple event tuples to Event structs.","ref":"AI.Stream.Transformer.html#convert_tuple_to_event/1"},{"type":"function","title":"Parameters - AI.Stream.Transformer.convert_tuple_to_event/1","doc":"* `tuple_event` - An event in tuple format (e.g., {:text_delta, content})","ref":"AI.Stream.Transformer.html#convert_tuple_to_event/1-parameters"},{"type":"function","title":"Returns - AI.Stream.Transformer.convert_tuple_to_event/1","doc":"* A corresponding AI.Stream.Event struct","ref":"AI.Stream.Transformer.html#convert_tuple_to_event/1-returns"},{"type":"callback","title":"AI.Stream.Transformer.transform/2","doc":"Transforms a provider-specific event stream into a stream of standardized events.\n\nTakes a source stream and any provider-specific options, and returns a new stream\nthat emits standardized AI.Stream.Event structs.","ref":"AI.Stream.Transformer.html#c:transform/2"},{"type":"callback","title":"Parameters - AI.Stream.Transformer.transform/2","doc":"* `source_stream` - The original stream from the provider API\n  * `options` - Additional options specific to this transformer (if any)","ref":"AI.Stream.Transformer.html#c:transform/2-parameters"},{"type":"callback","title":"Returns - AI.Stream.Transformer.transform/2","doc":"* A Stream that emits AI.Stream.Event structs","ref":"AI.Stream.Transformer.html#c:transform/2-returns"},{"type":"exception","title":"AI.Error","doc":"Standard error structure for AI module operations.\n\nThis module provides consistent error handling across the AI SDK\nwith helpful error messages and context.","ref":"AI.Error.html"},{"type":"function","title":"AI.Error.from_exception/2","doc":"Converts a standard exception to an AI.Error.\n\nThis is useful for standardizing error handling by converting\nvarious exception types to our common error format.","ref":"AI.Error.html#from_exception/2"},{"type":"function","title":"Parameters - AI.Error.from_exception/2","doc":"* `exception` - The exception to convert\n  * `source` - The module that caught the exception","ref":"AI.Error.html#from_exception/2-parameters"},{"type":"function","title":"Returns - AI.Error.from_exception/2","doc":"* `%AI.Error{}` - A new error struct","ref":"AI.Error.html#from_exception/2-returns"},{"type":"function","title":"AI.Error.new/1","doc":"Creates a new error with the given message.","ref":"AI.Error.html#new/1"},{"type":"function","title":"Parameters - AI.Error.new/1","doc":"* `message` - A string describing the error","ref":"AI.Error.html#new/1-parameters"},{"type":"function","title":"Returns - AI.Error.new/1","doc":"* `%AI.Error{}` - A new error struct","ref":"AI.Error.html#new/1-returns"},{"type":"function","title":"AI.Error.new/2","doc":"Creates a new error with the given message and reason.","ref":"AI.Error.html#new/2"},{"type":"function","title":"Parameters - AI.Error.new/2","doc":"* `message` - A string describing the error\n  * `reason` - The underlying reason or error detail","ref":"AI.Error.html#new/2-parameters"},{"type":"function","title":"Returns - AI.Error.new/2","doc":"* `%AI.Error{}` - A new error struct with reason","ref":"AI.Error.html#new/2-returns"},{"type":"function","title":"AI.Error.new/3","doc":"Creates a new error with the given message, reason, and source module.","ref":"AI.Error.html#new/3"},{"type":"function","title":"Parameters - AI.Error.new/3","doc":"* `message` - A string describing the error\n  * `reason` - The underlying reason or error detail\n  * `source` - The module that originated the error","ref":"AI.Error.html#new/3-parameters"},{"type":"function","title":"Returns - AI.Error.new/3","doc":"* `%AI.Error{}` - A new error struct with reason and source","ref":"AI.Error.html#new/3-returns"},{"type":"type","title":"AI.Error.t/0","doc":"","ref":"AI.Error.html#t:t/0"},{"type":"function","title":"AI.Error.wrap/3","doc":"Wraps an existing error in an AI.Error with additional context.\n\nThis is useful for adding context to lower-level errors while preserving\nthe original error information.","ref":"AI.Error.html#wrap/3"},{"type":"function","title":"Parameters - AI.Error.wrap/3","doc":"* `original_error` - The original error or exception\n  * `message` - A string describing the context of the error\n  * `source` - The module wrapping the error","ref":"AI.Error.html#wrap/3-parameters"},{"type":"function","title":"Returns - AI.Error.wrap/3","doc":"* `%AI.Error{}` - A new error struct wrapping the original error","ref":"AI.Error.html#wrap/3-returns"},{"type":"exception","title":"AI.Provider.UnsupportedFunctionalityError","doc":"Error raised when a functionality is not supported by the model.","ref":"AI.Provider.UnsupportedFunctionalityError.html"},{"type":"function","title":"AI.Provider.UnsupportedFunctionalityError.message/1","doc":"","ref":"AI.Provider.UnsupportedFunctionalityError.html#message/1"},{"type":"type","title":"AI.Provider.UnsupportedFunctionalityError.t/0","doc":"","ref":"AI.Provider.UnsupportedFunctionalityError.html#t:t/0"},{"type":"extras","title":"Elixir AI SDK","doc":"# Elixir AI SDK\n\nAn Elixir port of the [Vercel AI SDK](https://ai-sdk.dev/), designed to help you build AI-powered applications using Elixir and Phoenix.","ref":"readme.html"},{"type":"extras","title":"Features - Elixir AI SDK","doc":"- **Text Generation**: Generate text using Large Language Models (LLMs)\n- **Real-time Streaming**: Stream text responses chunk-by-chunk as they're generated\n- **Server-Sent Events (SSE)**: Efficient streaming with proper backpressure handling\n- **Tool Calling**: Call tools and functions from LLMs\n- **Provider Support**: Support for popular AI providers (OpenAI, Anthropic, etc.)\n- **Elixir Integration**: Seamless integration with Elixir's Stream API","ref":"readme.html#features"},{"type":"extras","title":"Installation - Elixir AI SDK","doc":"```elixir\ndef deps do\n  [\n    {:ai_sdk, \"~> 0.0.1-rc.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage Examples - Elixir AI SDK","doc":"","ref":"readme.html#usage-examples"},{"type":"extras","title":"Text Generation - Elixir AI SDK","doc":"```elixir\n{:ok, result} = AI.generate_text(%{\n  model: AI.openai(\"gpt-4o\"),\n  system: \"You are a friendly assistant!\",\n  prompt: \"Why is the sky blue?\"\n})\n\nIO.puts(result.text)\n```","ref":"readme.html#text-generation"},{"type":"extras","title":"Streaming Text Generation - Elixir AI SDK","doc":"```elixir\n# Default mode: returns a stream of text strings\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-4o\"),\n  system: \"You are a friendly assistant!\",\n  prompt: \"Why is the sky blue?\"\n})\n\n# Simple string processing\nresult.stream\n|> Stream.each(&IO.write/1)\n|> Stream.run()\n\n# Or collect all chunks\nfull_text = Enum.join(result.stream, \"\")\n```","ref":"readme.html#streaming-text-generation"},{"type":"extras","title":"Advanced Streaming with Event Mode - Elixir AI SDK","doc":"```elixir\n# Event mode: returns a stream of event tuples for fine-grained control\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-4o\"),\n  system: \"You are a friendly assistant!\",\n  prompt: \"Why is the sky blue?\",\n  mode: :event\n})\n\n# Process different event types\nEnum.reduce_while(result.stream, [], fn\n  # Text chunk received - print it and continue\n  {:text_delta, chunk}, acc ->\n    IO.write(chunk)\n    {:cont, [chunk | acc]}\n    \n  # Stream finished - print reason and halt collection\n  {:finish, reason}, acc ->\n    IO.puts(\"\\nFinished: #{reason}\")\n    {:halt, acc}\n    \n  # Error received - print it and halt collection\n  {:error, error}, acc ->\n    IO.puts(\"\\nError: #{inspect(error)}\")\n    {:halt, acc}\n    \n  # Other events - just continue\n  _, acc ->\n    {:cont, acc}\nend)\n```","ref":"readme.html#advanced-streaming-with-event-mode"},{"type":"extras","title":"Architecture - Elixir AI SDK","doc":"The Elixir AI SDK follows similar patterns to the original Vercel AI SDK but uses Elixir idioms and patterns:\n\n- Stream-based API for real-time text generation\n- Server-Sent Events (SSE) for efficient streaming\n- Proper backpressure handling for large responses\n- Elixir-native error handling with tagged tuples\n- Phoenix integration for web applications","ref":"readme.html#architecture"},{"type":"extras","title":"Development Status - Elixir AI SDK","doc":"This project is under active development.","ref":"readme.html#development-status"},{"type":"extras","title":"Documentation - Elixir AI SDK","doc":"- [Streaming Guide](docs/streaming.md) - Real-time text generation with SSE\n- Full API documentation available on [HexDocs](https://hexdocs.pm/ai_sdk)","ref":"readme.html#documentation"},{"type":"extras","title":"AI SDK - Streaming Capabilities","doc":"# AI SDK - Streaming Capabilities","ref":"streaming.html"},{"type":"extras","title":"Overview - AI SDK - Streaming Capabilities","doc":"The Elixir AI SDK provides streaming capabilities for text generation, allowing you to receive AI model responses incrementally as they're generated instead of waiting for the complete response. This provides a more responsive user experience, especially for longer responses.\n\nThe SDK implements Server-Sent Events (SSE) streaming using a robust EventSource implementation based on Finch to efficiently process responses chunk by chunk in real-time with minimal memory overhead. This implementation properly handles backpressure, connection management, and parsing of the SSE protocol.","ref":"streaming.html#overview"},{"type":"extras","title":"Streaming Modes - AI SDK - Streaming Capabilities","doc":"The SDK supports two streaming modes to accommodate different use cases:\n\n1. **String Mode (Default)**: Returns a stream of plain text chunks, ideal for simple use cases\n2. **Event Mode**: Returns a stream of event tuples for fine-grained control over different event types","ref":"streaming.html#streaming-modes"},{"type":"extras","title":"Getting Started with Streaming - AI SDK - Streaming Capabilities","doc":"","ref":"streaming.html#getting-started-with-streaming"},{"type":"extras","title":"String Mode (Default) - AI SDK - Streaming Capabilities","doc":"The default mode returns a stream of text strings, making it extremely simple to use:\n\n```elixir\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  prompt: \"Write a short story about a robot learning to paint.\"\n})\n\n# Process the stream - each chunk is a simple string\nresult.stream\n|> Stream.each(&IO.write/1)  # Write each chunk as it arrives\n|> Stream.run()  # Start consuming the stream\n\n# Or collect all chunks into a single string\nfull_text = Enum.join(result.stream, \"\")\n```","ref":"streaming.html#string-mode-default"},{"type":"extras","title":"Event Mode - AI SDK - Streaming Capabilities","doc":"For applications that need to handle different types of streaming events (text, errors, completion), use event mode:\n\n```elixir\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  prompt: \"Write a short story about a robot learning to paint.\",\n  mode: :event  # Enable event mode\n})\n\n# Process different event types\nresult.stream\n|> Enum.each(fn\n  {:text_delta, chunk} -> IO.write(chunk)\n  {:finish, reason} -> IO.puts(\"\\nCompleted: #{reason}\")\n  {:error, error} -> IO.puts(\"\\nError: #{inspect(error)}\")\n  _ -> :ok  # Ignore other events\nend)\n```","ref":"streaming.html#event-mode"},{"type":"extras","title":"Stream Format - AI SDK - Streaming Capabilities","doc":"","ref":"streaming.html#stream-format"},{"type":"extras","title":"String Mode Format - AI SDK - Streaming Capabilities","doc":"In string mode, the stream produces text chunks directly as strings. Each chunk is a fragment of the model's response, making it easy to consume and work with.","ref":"streaming.html#string-mode-format"},{"type":"extras","title":"Event Mode Format - AI SDK - Streaming Capabilities","doc":"In event mode, the stream produces tuples representing different event types:\n\n- `{:text_delta, String.t()}` - A chunk of text from the model\n- `{:finish, String.t()}` - Stream has completed with reason (e.g., \"stop\", \"length\")\n- `{:metadata, map()}` - Additional metadata from the model\n- `{:error, term()}` - An error occurred during streaming\n\nThese events are produced by parsing the Server-Sent Events (SSE) format returned by AI providers. The SDK handles all the complexities of SSE parsing, including multi-line data fields, JSON parsing, and event formatting.","ref":"streaming.html#event-mode-format"},{"type":"extras","title":"Examples - AI SDK - Streaming Capabilities","doc":"","ref":"streaming.html#examples"},{"type":"extras","title":"Basic Streaming - AI SDK - Streaming Capabilities","doc":"```elixir\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  prompt: \"Explain how streaming works in LLMs.\"\n})\n\nresult.stream\n|> Stream.each(&IO.write/1)\n|> Stream.run()\n```","ref":"streaming.html#basic-streaming"},{"type":"extras","title":"With System Message - AI SDK - Streaming Capabilities","doc":"```elixir\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  system: \"You are a helpful assistant that responds in the style of Shakespeare.\",\n  prompt: \"Tell me about the weather today.\"\n})\n\nresult.stream\n|> Stream.each(&IO.write/1)\n|> Stream.run()\n```","ref":"streaming.html#with-system-message"},{"type":"extras","title":"Collecting the Full Response - AI SDK - Streaming Capabilities","doc":"If you want to collect all chunks into a single string:\n\n```elixir\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  prompt: \"Write a haiku about programming.\"\n})\n\nfull_text = Enum.join(result.stream, \"\")\n\nIO.puts(\"Full response: #{full_text}\")\n```","ref":"streaming.html#collecting-the-full-response"},{"type":"extras","title":"Error Handling - AI SDK - Streaming Capabilities","doc":"```elixir\ncase AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  prompt: \"Tell me a joke.\"\n}) do\n  {:ok, result} ->\n    # Simply process the text chunks\n    result.stream\n    |> Stream.each(&IO.write/1)\n    |> Stream.run()\n\n  {:error, error} ->\n    IO.puts(\"Failed to start streaming: #{inspect(error)}\")\nend\n```\n\nThe SDK handles several types of errors during streaming:\n- Initial connection failures\n- Network interruptions during streaming\n- Invalid SSE format\n- Timeouts (default: 30 seconds for initial connection)\n- HTTP error responses from the provider\n- Finch-related errors\n- JSON parsing errors from malformed responses\n\nAll error conditions are properly propagated as `{:error, reason}` events in the stream or as an error response from the initial connection.","ref":"streaming.html#error-handling"},{"type":"extras","title":"Advanced Usage - AI SDK - Streaming Capabilities","doc":"","ref":"streaming.html#advanced-usage"},{"type":"extras","title":"EventSource Implementation - AI SDK - Streaming Capabilities","doc":"The SDK uses a custom `AI.Provider.Utils.EventSource` module that implements the Server-Sent Events (SSE) protocol. This implementation:\n\n- Creates and manages HTTP connections with proper headers\n- Parses the SSE format according to specification\n- Handles event reassembly from chunks\n- Provides proper backpressure for efficient streaming\n- Cleans up resources when the stream is done","ref":"streaming.html#eventsource-implementation"},{"type":"extras","title":"Custom Stream Processing - AI SDK - Streaming Capabilities","doc":"You can use all of Elixir's stream processing capabilities:\n\n```elixir\n{:ok, result} = AI.stream_text(%{\n  model: AI.openai(\"gpt-3.5-turbo\"),\n  prompt: \"List 10 programming languages.\"\n})\n\nresult.stream\n|> Stream.chunk_by(fn chunk -> chunk == \"\\n\" end)\n|> Stream.reject(fn chunk -> chunk == [\"\\n\"] end)\n|> Stream.map(fn chunks -> Enum.join(chunks, \"\") end)\n|> Stream.each(fn line -> IO.puts(\"Language: #{line}\") end)\n|> Stream.run()\n```","ref":"streaming.html#custom-stream-processing"},{"type":"extras","title":"Working with Phoenix LiveView - AI SDK - Streaming Capabilities","doc":"Streaming works especially well with Phoenix LiveView, allowing you to update the UI in real-time as responses are generated:\n\n```elixir\ndef handle_event(\"generate\", %{\"prompt\" => prompt}, socket) do\n  # Start streaming in a separate process\n  Task.async(fn ->\n    case AI.stream_text(%{\n      model: AI.openai(\"gpt-3.5-turbo\"),\n      prompt: prompt\n    }) do\n      {:ok, result} ->\n        result.stream\n        |> Stream.each(fn chunk ->\n            # Send each chunk to the LiveView process\n            send(self(), {:stream_chunk, chunk})\n        end)\n        |> Stream.run()\n        \n        send(self(), :stream_complete)\n        \n      {:error, error} ->\n        send(self(), {:stream_error, error})\n    end\n  end)\n  \n  {:noreply, socket |> assign(generating: true, response: \"\")}\nend\n\ndef handle_info({:stream_chunk, chunk}, socket) do\n  # Append the new chunk to the existing response\n  new_response = socket.assigns.response <> chunk\n  {:noreply, socket |> assign(response: new_response)}\nend\n\ndef handle_info(:stream_complete, socket) do\n  {:noreply, socket |> assign(generating: false)}\nend\n\ndef handle_info({:stream_error, error}, socket) do\n  {:noreply, socket |> assign(generating: false, error: inspect(error))}\nend\n```","ref":"streaming.html#working-with-phoenix-liveview"},{"type":"extras","title":"API Reference - AI SDK - Streaming Capabilities","doc":"### `AI.stream_text/1`\n\n```elixir\n@spec stream_text(map()) :: {:ok, map()} | {:error, any()}\n```\n\nOptions:\n\n* `:model` - The language model to use\n* `:system` - A system message that will be part of the prompt\n* `:prompt` - A simple text prompt (can use either prompt or messages)\n* `:messages` - A list of messages (can use either prompt or messages)\n* `:max_tokens` - Maximum number of tokens to generate\n* `:temperature` - Temperature setting for randomness\n* `:top_p` - Nucleus sampling\n* `:top_k` - Top-k sampling\n* `:frequency_penalty` - Penalize new tokens based on their frequency\n* `:presence_penalty` - Penalize new tokens based on their presence\n* `:tools` - Tools that are accessible to and can be called by the model\n* `:mode` - Stream output format: `:string` (default) or `:event`\n\nStreaming-specific options:\n* `:timeout` - Connection timeout in milliseconds (default: 30000)\n* `:max_line_length` - Maximum length of an SSE line (default: 16384)\n* `:retry_interval` - Time to wait before reconnecting in ms (default: 3000)\n* `:test_mode` - For testing only, can be `:basic`, `:openai`, `:multi_line`, or `:error`\n\nReturns:\n\n* `{:ok, result}` - Success, with result containing:\n  * `stream` - The stream of text chunks (strings in string mode, event tuples in event mode)\n  * `warnings` - Any warnings generated during processing\n  * `provider_metadata` - Additional provider-specific metadata\n  * `response` - The raw response from the provider\n\n* `{:error, reason}` - Error with reason for failure","ref":"streaming.html#api-reference"},{"type":"extras","title":"Limitations and Future Improvements - AI SDK - Streaming Capabilities","doc":"* Tool calls via streaming are not yet fully supported\n* Support for structured output streaming is planned for future releases\n* Future enhancements will include:\n  * Improved backpressure handling for very large responses\n  * Automatic reconnection for temporary network issues\n  * Progress tracking and statistics\n  * Configurable stream processing pipelines\n  * Additional providers beyond OpenAI","ref":"streaming.html#limitations-and-future-improvements"},{"type":"extras","title":"Technical Implementation - AI SDK - Streaming Capabilities","doc":"The streaming functionality is built on several key components:\n\n1. `AI.stream_text/1` - The main API entry point\n2. `AI.Core.StreamText` - Core implementation of stream handling\n3. `AI.Provider.Utils.EventSource` - SSE protocol implementation\n4. Provider-specific `do_stream` implementations (e.g., OpenAI)\n\nThe SSE protocol implementation follows the [HTML5 EventSource specification](https://html.spec.whatwg.org/multipage/server-sent-events.html) and is compatible with the streaming APIs of major providers like OpenAI.","ref":"streaming.html#technical-implementation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}